@@ -2,26 +2,57 @@ package My::Build::Base;
 
 use strict;
 use base qw(Module::Build);
-use My::Build::Utility qw(awx_arch_file);
+use My::Build::Utility qw(awx_arch_file awx_touch);
 use Alien::wxWidgets::Utility qw(awx_sort_config awx_grep_config);
 use File::Path ();
 use File::Basename ();
-use Fatal qw(open close);
+use Fatal qw(open close unlink);
 use Data::Dumper;
+use File::Glob qw(bsd_glob);
+
+# use the system version of a module if present; in theory this could lead to
+# compatibility problems (if the latest version of one of the dependencies,
+# installed in @INC is incompatible with the bundled version of a module)
+sub _load_bundled_modules {
+    # the load order is important: all dependencies must be loaded
+    # before trying to load a module
+    require inc::latest;
+
+    inc::latest->import( $_ )
+        foreach qw(version
+                   Locale::Maketext::Simple
+                   Params::Check
+                   Module::Load
+                   Module::Load::Conditional
+                   IPC::Cmd
+                   Archive::Extract
+                   File::Fetch);
+}
 
 sub ACTION_build {
     my $self = shift;
-
+    # try to make "perl Makefile.PL && make test" work
+    # but avoid doubly building wxWidgets when doing
+    # "perl Makefile.PL && make && make test"
+    unlink 'configured' if -f 'configured';
     $self->SUPER::ACTION_build;
-    if( $self->notes( 'build_wx' ) ) {
-        $self->fetch_wxwidgets;
-        $self->extract_wxwidgets;
-        $self->massage_environment;
-        $self->build_wxwidgets;
-        $self->massage_environment; # twice on purpose
-    }
+}
+
+sub ACTION_code {
+    my $self = shift;
+
+    $self->SUPER::ACTION_code;
+    # install_only is set when a wxWidgets build is already configured
+    # with Alien::wxWidgets
+    return if $self->notes( 'install_only' );
+    # see comment in ACTION_build for why 'configured' is used
+    return if -f 'configured';
+    $self->depends_on( 'build_wx' );
     $self->create_config_file( awx_arch_file( 'Config/Config.pm' ) );
     $self->install_wxwidgets;
+    # see comment in ACTION_build for why 'configured' is used
+    awx_touch( 'configured' );
+    $self->add_to_cleanup( 'configured' );
 }
 
 sub ACTION_build_wx {
@@ -58,9 +89,20 @@ sub ACTION_install {
     $self->install_system_wxwidgets;
 }
 
-sub ACTION_distcheck {
-    my $self = shift;
-    my $data = $self->notes( 'build_data' );
+sub _check_data_file {
+    my( $self, $file, $manifest ) = @_;
+
+    require File::Spec::Unix;
+
+    my $data = do {
+        package main;
+        our( $TYPE, $URL );
+        local $TYPE = 'dummy';
+        local $URL = 'dummy';
+
+        do $file;
+    };
+    die "Unable to load data file '$file': $@" unless $data;
 
     foreach my $p ( qw(msw mac unix) ) {
         next unless exists $data->{$p};
@@ -70,13 +112,40 @@ sub ACTION_distcheck {
 
             foreach my $f ( @{$data->{$p}{$c}} ) {
                 my $file = File::Spec->catfile( 'patches', $f );
+                my $manifest_file = File::Spec::Unix->catfile( 'patches', $f );
 
-                warn 'Missing patch file: ', $file, "\n" unless -f $file;
+                die 'Missing patch file: ', $file, "\n" unless -f $file;
+                die 'Patch file ', $file, ' not in MANIFEST'
+                  unless exists $manifest->{$manifest_file};
             }
         }
     }
+}
+
+sub _check_data_files {
+    my( $self ) = @_;
+
+    require ExtUtils::Manifest;
+    my $files = ExtUtils::Manifest::maniread();
+
+    foreach my $data ( grep m{^patches/data}, keys %$files ) {
+        print "Checking $data\n";
+        $self->_check_data_file( $data, $files );
+    }
+}
+
+sub ACTION_distcheck {
+    my $self = shift;
 
     $self->SUPER::ACTION_distcheck;
+    $self->_check_data_files;
+}
+
+sub ACTION_dist {
+    my $self = shift;
+
+    $self->_check_data_files;
+    $self->SUPER::ACTION_dist;
 }
 
 sub awx_key {
@@ -100,7 +169,7 @@ sub _version_2_dec {
     $ver =~ m/^(\d+)\.(\d+)\.(\d+)$/ and
       $dec = $1 + $2 / 1000 + $3 / 1000000;
 
-    return $dec;
+    return sprintf( "%.6f", $dec );
 }
 
 sub _init_config {
@@ -144,17 +213,6 @@ sub _init_config {
 sub create_config_file {
     my( $self, $file ) = @_;
 
-=pod
-
-    if( -f 'configured' ) {
-        warn "Remove 'configured' to reconfigure wxWidgets";
-        my $config = do 'configured';
-        $self->{awx_base} = $self->{awx_key} = $config->{alien_base};
-        return;
-    }
-
-=cut
-
     my $directory = File::Basename::dirname( $file );
     my %config = $self->_init_config;
     my $base = $self->awx_key;
@@ -188,6 +246,35 @@ EOT
 }
 
 my $key = substr __PACKAGE__, 1 + rindex __PACKAGE__, ':';
+EOT
+
+    print $fh <<'EOT' if $self->notes( 'mk_portable' ) && ( $^O =~ /^MSWin/ );
+
+my ($portablebase);
+my $wxwidgetspath = __PACKAGE__ . '.pm';
+$wxwidgetspath =~ s/::/\//g;
+
+for (@INC) {
+    if( -f qq($_/$wxwidgetspath ) ) {
+        $portablebase = qq($_/Alien/wxWidgets/$key);
+        last;
+    }
+}
+
+if( $portablebase ) {
+    $portablebase =~ s{/}{\\}g;
+    my $portablelibpath = qq($portablebase\\lib);
+    my $portableincpath = qq($portablebase\\include);
+
+    $VALUES{include_path} = qq{-I$portablelibpath -I$portableincpath};
+    $VALUES{link_libraries} =~ s{-L\S+\s}{-L$portablelibpath };
+    $VALUES{shared_library_path} = $portablelibpath;
+    $VALUES{wx_base_directory} = $portablebase;
+    $VALUES{prefix} = $portablebase;
+}
+EOT
+
+    print $fh <<'EOT';
 
 sub values { %VALUES, key => $key }
 
@@ -203,24 +290,13 @@ sub config {
 EOT
 
     close $fh;
-
-=pod
-
-    $self->add_to_cleanup( 'configured' );
-    {
-        open my $fh, '>', 'configured';
-        print $fh $body;
-    }
-
-=cut
-
 }
 
 sub fetch_wxwidgets {
     my $self = shift;
 
     return if -f $self->notes( 'build_data' )->{data}{archive};
-    require File::Fetch;
+    $self->_load_bundled_modules;
 
     print "Fetching wxWidgets...\n";
     print "fetching from: ", $self->notes( 'build_data' )->{data}{url}, "\n";
@@ -238,7 +314,8 @@ sub extract_wxwidgets {
 
     print "Extracting wxWidgets...\n";
 
-    require Archive::Extract;
+    $self->_load_bundled_modules;
+    $Archive::Extract::PREFER_BIN = 1;
     my $ae = Archive::Extract->new( archive => $archive );
 
     die 'Error: ', $ae->error unless $ae->extract;
@@ -253,12 +330,13 @@ sub patch_wxwidgets {
 
     print "Patching wxWidgets...\n";
 
-    chdir File::Spec->rel2abs
-              ( $self->notes( 'build_data' )->{data}{directory} );
+    my $wx_dir = $self->notes( 'build_data' )->{data}{directory};
+    my $build_dir = File::Spec->rel2abs( $wx_dir );
+    chdir $wx_dir;
 
     foreach my $i ( @patches ) {
         print "Applying patch: ", $i, "\n";
-        my $cmd = $self->_patch_command( $old_dir, $i );
+        my $cmd = $self->_patch_command( $build_dir, $i );
         print $cmd, "\n";
         system $cmd and die 'Error: ', $?;
     }
@@ -269,7 +347,8 @@ sub patch_wxwidgets {
 sub _patch_command {
     my( $self, $base_dir, $patch_file ) = @_;
 
-    my $cmd = $^X . ' ' . File::Spec->catfile( $base_dir,
+    $patch_file = File::Spec->abs2rel( $patch_file, $base_dir );
+    my $cmd = $^X . ' ' . File::Spec->catfile( File::Spec->updir,
                                                qw(inc bin patch) )
       . " -N -p0 -u -b .bak < $patch_file";
 
@@ -296,7 +375,7 @@ sub awx_configure {
     $config{config}{debug} = $self->awx_is_debug;
     $config{config}{unicode} = $self->awx_is_unicode;
     $config{config}{mslu} = $self->awx_is_mslu;
-    $config{config}{build} = 'multi';
+    $config{config}{build} = $self->awx_is_monolithic ? 'mono' : 'multi';
     $config{link_flags} = '';
     $config{c_flags} = '';
 
@@ -315,16 +394,19 @@ sub wx_config {
     return @{$data}{@_};
 }
 
-sub awx_monolithic { $_[0]->args( 'monolithic' ) ? 1 : 0 }
+sub awx_monolithic { $_[0]->args( 'wxWidgets-monolithic' ) ? 1 : 0 }
 sub awx_is_monolithic { $_[0]->awx_monolithic }
-sub awx_debug { $_[0]->args( 'debug' ) ? 1 : 0 }
+sub awx_debug { $_[0]->args( 'wxWidgets-debug' ) ? 1 : 0 }
 sub awx_is_debug { $_[0]->awx_debug }
-sub awx_unicode { $_[0]->args( 'unicode' ) ? 1 : 0 }
+sub awx_unicode { $_[0]->notes( 'build_wx_unicode' )
+                    || $_[0]->args( 'wxWidgets-unicode' ) ? 1 : 0 }
 sub awx_is_unicode { $_[0]->awx_unicode }
 sub awx_mslu { 0 }
 sub awx_is_mslu { $_[0]->awx_mslu }
-sub awx_static { $_[0]->args( 'static' ) ? 1 : 0 }
+sub awx_static { $_[0]->args( 'wxWidgets-static' ) ? 1 : 0 }
 sub awx_is_static { $_[0]->awx_static }
+sub awx_universal { $_[0]->args( 'wxWidgets-universal' ) ? 1 : 0 }
+sub awx_is_universal { $_[0]->awx_universal }
 sub awx_get_package { local $_ = $_[0]; s/^My::Build:://; return $_ }
 
 sub awx_wx_patches {
@@ -370,7 +452,10 @@ sub awx_path_search {
 
     foreach my $d ( File::Spec->path ) {
         my $full = File::Spec->catfile( $d, $file );
-        return $full if -f $full;
+        # we are gonna use glob() to accept wildcards
+        foreach my $f ( bsd_glob( $full ) ) {
+            return $f if -f $f;
+        }
     }
 
     return;
